import socialmedia.AccountIDNotRecognisedException;
import socialmedia.MiniSocialMedia;
import socialmedia.MiniSocialMediaPlatform;
import socialmedia.IllegalHandleException;
import socialmedia.InvalidHandleException;
import socialmedia.HandleNotRecognisedException;
import socialmedia.InvalidPostException;
import socialmedia.PostIDNotRecognisedException;
import socialmedia.NotActionablePostException;
import java.io.*;

/**
 * A short program to illustrate an app testing some minimal functionality of a
 * concrete implementation of the SocialMediaPlatform interface -- note you will
 * want to increase these checks, and run it on your SocialMedia class (not the
 * BadSocialMedia class).
 *
 * 
 * @author Diogo Pacheco
 * @version 1.0
 */
public class SocialMediaPlatformTestApp {

	/**
	 * Test method.
	 * 
	 * @param args not used
	 */
	public static void main(String[] args) {
		System.out.println("The system compiled and started the execution...");

		MiniSocialMediaPlatform platform = new MiniSocialMedia();

		boolean isPassed = true;
		
		System.out.println();
		
		try {
			System.out.println("Perform platform.createAccount...");
			int id = platform.createAccount("test_handle");
			
			System.out.println("Perform platform.removeAccount...");
			platform.removeAccount(id);
			
			System.out.println("Perform platform.changeAccountHandle...");
			id = platform.createAccount("test_handle");
			platform.changeAccountHandle("test_handle", "new_handle");
			
			System.out.println("Perform platform.showAccount...");
			String accountInfo = platform.showAccount("new_handle");
			
			String expectedAccountInfo = "ID: 1\nHandle: new_handle\nDescription: \nPost count: 0\nEndorse count: 0";
			
			if(!accountInfo.equals(expectedAccountInfo)) {
				isPassed = false;
				System.out.println("AccountInfo is not matching with expectation.");
			}
			
			System.out.println("Perform platform.createPost...");
			int postID = platform.createPost("new_handle", "new_message");
			
			System.out.println("Perform platform.endorsePost...");
			int epostID = platform.endorsePost("new_handle", postID);
			
			System.out.println("Perform platform.commentPost...");
			int cpostID = platform.commentPost("new_handle", postID, "random_comment");
			
			System.out.println("Perform platform.showIndividualPost...");
			String individualPost = platform.showIndividualPost(postID);
			
			String expectedIndividualPost = "ID: 0\nAccount: new_handle\nNo. endorsements: 1 | No. comments: 1\nnew_message\n";
			
			System.out.println("IndividualPost :\n" + individualPost);
			
			if(!individualPost.equals(expectedIndividualPost)) {
				isPassed = false;
				System.out.println("showIndividualPost is not matching with expectation.");
			}
			
			System.out.println("Perform platform.showPostChildrenDetails...");
			StringBuilder postChildrenDetailsSB = platform.showPostChildrenDetails(postID);
			String postChildrenDetails = postChildrenDetailsSB.toString();
			
			String expectedPostChildrenDetails = "ID: 0\nAccount: new_handle\nNo. endorsements: 1 | No. comments: 1\nnew_message\n|\n| > ID: 2\n    Account: new_handle\n    No. endorsements: 0 | No. comments: 0\n    random_comment\n";
			
			System.out.println("PostChildrenDetails :\n" + postChildrenDetails);
			
			if(!postChildrenDetails.equals(expectedPostChildrenDetails)) {
				isPassed = false;
				System.out.println("showPostChildrenDetails is not matching with expectation.");
			}
			
			System.out.println("Perform platform.getMostEndorsedPost...");
			int mostEndoredPostID = platform.getMostEndorsedPost();
			
			if(mostEndoredPostID != 0) {
				isPassed = false;
				System.out.println("getMostEndorsedPost is not matching with expectation.");
			}
			
			System.out.println("Perform platform.getMostEndorsedAccount...");
			int mostEndoredAccount = platform.getMostEndorsedAccount();
			
			if(mostEndoredAccount != 1) {
				isPassed = false;
				System.out.println("getMostEndorsedAccount is not matching with expectation.");
			}
			
			System.out.println("Perform platform.savePlatform...");
			platform.savePlatform("platform.ser");
			
			System.out.println("Perform platform.deletePost...");
			platform.deletePost(postID);
			
			System.out.println("Perform platform.getMostEndorsedPost...");
			mostEndoredPostID = platform.getMostEndorsedPost();
			
			if(mostEndoredPostID != -1) {
				isPassed = false;
				System.out.println("getMostEndorsedPost is not matching with expectation.");
			}
			
			System.out.println("Perform platform.getMostEndorsedAccount...");
			mostEndoredAccount = platform.getMostEndorsedAccount();
			
			if(mostEndoredAccount != -1) {
				isPassed = false;
				System.out.println("getMostEndorsedAccount is not matching with expectation.");
			}
			
			System.out.println("Perform platform.erasePlatform...");
			platform.erasePlatform();
			
			boolean expectedException = false;
			try {
				System.out.println("Perform platform.showAccount...");
				accountInfo = platform.showAccount("new_handle");
			} catch (HandleNotRecognisedException e) {
				expectedException = true;
			}
			
			if(!expectedException) {
				isPassed = false;
				System.out.println("HandleNotRecognisedException not thrown");
			}
			
			System.out.println("Perform platform.loadPlatform...");
			platform.loadPlatform("platform.ser");
			
			System.out.println("Perform platform.showAccount...");
			accountInfo = platform.showAccount("new_handle");
			
			expectedAccountInfo = "ID: 1\nHandle: new_handle\nDescription: \nPost count: 3\nEndorse count: 0";
			
			if(!accountInfo.equals(expectedAccountInfo)) {
				isPassed = false;
				System.out.println("AccountInfo is not matching with expectation.");
			}
			
		} catch (IllegalHandleException e) {
			isPassed = false;
			System.out.println("IllegalHandleException thrown incorrectly");
		} catch (InvalidHandleException e) {
			isPassed = false;
			System.out.println("InvalidHandleException thrown incorrectly");
		} catch (AccountIDNotRecognisedException e) {
			isPassed = false;
			System.out.println("AccountIDNotRecognizedException thrown incorrectly");
		} catch (HandleNotRecognisedException e) {
			isPassed = false;
			System.out.println("HandleNotRecognisedException thrown incorrectly");
		} catch (InvalidPostException e) {
			isPassed = false;
			System.out.println("InvalidPostException thrown incorrectly");
		} catch (PostIDNotRecognisedException e) {
			isPassed = false;
			System.out.println("PostIDNotRecognisedException thrown incorrectly");
		} catch (NotActionablePostException e) {
			isPassed = false;
			System.out.println("NotActionablePostException thrown incorrectly");
		} catch(IOException ex) {
            isPassed = false;
			System.out.println("IOException thrown incorrectly");
        } catch(ClassNotFoundException ex) {
            isPassed = false;
			System.out.println("ClassNotFoundException thrown incorrectly");
        }
		
		if(isPassed)
		{
			System.out.println();
			System.out.println("All Test Cases Passed.");
		}
	}

}
