package socialmedia;

/**
 * Post class
 *
 * 
 * @author 
 * @version 1.0
 */
import java.io.IOException;
import java.util.*;

class Post implements java.io.Serializable{
	public static int MAX_POST_ID = 0;
	public int ID;
	public String message;
	public String Handle;
	
	public ArrayList<Integer> endorsements;
	public ArrayList<Integer> comments;
	
	//Constructor
	public Post() {
		endorsements = new ArrayList<>();
		comments = new ArrayList<>();
	}
	
	//Helper method
	private void addExtraSpace(StringBuilder result, int level) {
		if(level == 0) return;
		
		for(int i = 0; i < level; i++) {
			result.append("    ");
		}
	}
	
	/**
	* Method prints the Individual Post details.
	*
	* @param id ID of the post
	* @param level for indentation of extra spaces
	*/
	public String showIndividualPost(int id, int level) {
		StringBuilder result = new StringBuilder();
				
		int commentsCount = comments.size(), 
					endorseCount = endorsements.size();
		
		if(level > 0)
		{
			addExtraSpace(result, level - 1);
			result.append("| > ");
		}
		
		result.append(String.format("ID: %d\n", ID));
		
		addExtraSpace(result, level);
		result.append(String.format("Account: %s\n", Handle));
		addExtraSpace(result, level);
		result.append(String.format("No. endorsements: %d | No. comments: %d\n", 
									endorseCount, commentsCount));
		addExtraSpace(result, level);
		result.append(String.format("%s\n", message));
		
		return result.toString();
	}
}

/**
 * EndorsementPost class
 *
 * 
 * @author 
 * @version 1.0
 */
class EndorsementPost extends Post implements java.io.Serializable{	
	public String referenceHandle;
	public int referencePostID;
}

/**
 * CommentPost class derived class of Post
 *
 * 
 * @author 
 * @version 1.0
 */
class CommentPost extends Post implements java.io.Serializable{	
	public String referenceHandle;
	public int referencePostID;
}


/**
 * Account class derived class of Post
 *
 * 
 * @author 
 * @version 1.0
 */
public class Account implements java.io.Serializable{
	public static int MAX_ID = 0;
	public int ID;
	public String Handle;
	public String Description;
	
	public HashMap<Integer, Post> posts;
	
	//Constructor
	public Account() {
		posts = new HashMap<>();
	}
	
	/**
	* Method adds a post into the account.
	*
	* @param id message of the post
	*/
	public int addPost(String message) {
		Post post = new Post();
		post.ID = Post.MAX_POST_ID++;
		post.message = message;
		post.Handle = Handle;
		this.posts.put(post.ID, post);
		
		return post.ID;
	}
	
	/**
	* Method adds a endorsement to a post of the account.
	*
	* @param handle of the account
	* @param refID of the Post
	*/
	public int addEndorsementPost(String handle, int refID) {
		EndorsementPost epost = new EndorsementPost();
		epost.ID = Post.MAX_POST_ID++;
		epost.referenceHandle = handle;
		epost.referencePostID = refID;
		epost.Handle = Handle;
		
		this.posts.put(epost.ID, epost);
		
		return epost.ID;
	}
	
	/**
	* Method adds a comment to a post of the account.
	*
	* @param handle of the account
	* @param refID of the Post
	* @param commentMessage of the Comment
	*/
	public int addCommentPost(String handle, int refID, String commentMessage) {
		CommentPost cpost = new CommentPost();
		cpost.ID = Post.MAX_POST_ID++;
		cpost.referenceHandle = handle;
		cpost.Handle = Handle;
		cpost.message = commentMessage;
		cpost.referencePostID = refID;
		
		this.posts.put(cpost.ID, cpost);
		
		return cpost.ID;
	}
	
	/**
	* Method checks if the post ID in current account.
	*
	* @param ID of the Post
	*/
	public boolean containsPost(int id) {
		return this.posts.containsKey(id);
	}
	
	/**
	* Method checks if the post is Endorsement.
	*
	* @param ID of the Post
	*/
	public boolean isEndorsementPostPost(int id) {
		Post e = this.posts.get(id);
		
		if(e instanceof EndorsementPost) return true;
		
		return false;
	}
	
	/**
	* Method adds a EndorsementPost to a post of the account.
	*
	* @param handle of the account
	* @param id of the Post
	* @param eid of the EndorsementPost
	*/
	public boolean endorsePost(String handle, int id, int eid) {
		Post e = this.posts.get(id);
		
		if(e instanceof EndorsementPost) return false;
		
		e.endorsements.add(eid);
		return true;
	}
	
	/**
	* Method adds a CommentPost to a post of the account.
	*
	* @param handle of the account
	* @param id of the Post
	* @param cid of the CommentPost
	*/
	public boolean commentPost(String handle, int id, int cid) {
		Post e = this.posts.get(id);
		
		if(e instanceof EndorsementPost) return false;
		
		e.comments.add(cid);
		return true;
	}
	
	/**
	* Method delete the post.
	*
	* @param ID of the Post
	*/
	public boolean deletePost(int id) {
		
		if(!this.posts.containsKey(id)) return false;
		
		this.posts.remove(id);
		
		return true;
	}
	
	/**
	* Method prints the Individual Post details.
	*
	* @param id ID of the post
	* @param level for indentation of extra spaces
	*/
	public String showIndividualPost(int id, int level) {
		StringBuilder result = new StringBuilder();
		
		Post e = this.posts.get(id);
		
		return e.showIndividualPost(id, level);
	}
}